<?php

class Teamnet_Fax_Quickstart_Model {

	private $configuration;
	private $configurationFile;
	private $definitions;
	private $qsVersionInfo;
	private $qsService;
	private $diagnostics;
	
	const FILE_CONFIG = 'quickstart.ini';
	// This is the intended pattern, but does not work with php 5.1 (Be aware: If this was changes back, indexes of matches have to be adapted).
	//const PATTERN_DEFINE = '/define\\s*\\(\\s*(\'|")(.*?)\\g1\\s*,\\s*(\'|")(.*?)\\g3\\s*\\)\\s*;/';
	const PATTERN_DEFINE = '/define\\s*\\(\\s*(?:\'|")(.*?)(?:\'|")\\s*,\\s*(?:\'|")(.*?)(?:\'|")\\s*\\)\\s*;/';
	
	/**
	 * Enter description here ...
	 * @param array[string=>mixed]
	 */
	public function __construct( $configFile = self::FILE_CONFIG ) {
		$this->configurationFile = $configFile;
		$this->readConfiguration();
		$this->readDefinitions();
		
		$config = &$this->getConfiguration();
		if ( array_key_exists( 'diagnostics', $_SESSION )  && ! empty( $_SESSION['diagnostics'] ) ) {
			$this->diagnostics = $_SESSION['diagnostics'];
		}
		else {
			$this->diagnostics = new Teamnet_Fax_QuickStart_Diagnostics( $config );
			$_SESSION['diagnostics'] = $this->diagnostics;
		}
	}
	
	/**
	 * @return the $config
	 */
	public function &getConfiguration() {
		return $this->configuration;
	}

	/**
	 * @return Teamnet_Fax_Soap_Client_Quickstart
	 */
	public function getQsService() {
		if ( is_null( $this->qsService ) ) {
			$diagnostics = $this->getDiagnostics();
			$endpoint = $diagnostics->getEndpointGroup( $diagnostics->getSelectedEndpointGroupId() )->getEndpoint( 'Quickstart' );
			// do not used stored soap availability test result, because diagnostics may not run yet.
			if ( ! $diagnostics->testSoap() ) {
				throw new ServiceException( 'Soap client unavailable.' );
			}
			$this->qsService = new Teamnet_Fax_Soap_Client_Quickstart( $endpoint->getServiceDescriptor()->__toString() );
		}
		return $this->qsService;
	}

	/**
	 * Get version info about quickstart
	 * @return CheckVersionResponse
	 */
	public function getQsVersionInfo() {
		$config = &$this->getConfiguration();
		if ( is_null( $this->qsVersionInfo ) ) {
			try {
				$qsService = $this->getQsService();
				$qsVersionInfo = $qsService->checkVersion( $config['Quickstart']['Version'] );
			}
			catch ( ServiceException $exception ) {
				//Ignore non reachable quickstart service for update checks
				$qsVersionInfo = new CheckVersionResponse;
				$qsVersionInfo->actualVersion = $config['Quickstart']['Version'];
				$qsVersionInfo->versionStatus = 'unknown';
			}
		}
		return $qsVersionInfo;
	}
	
	public function postponeUpdate() {
		$qsVersionInfo = $this->getQsVersionInfo();
		$config = &$this->getConfiguration();
		$config['Quickstart']['UpdatePostponed'] = $qsVersionInfo->actualVersion;
		$this->writeConfiguration();
	}
	
	public function isUpdateAvailable() {
		$config = &$this->getConfiguration();
		$qsConfig = $config['Quickstart'];
		$qsVersionInfo = $this->getQsVersionInfo();
		$updateAvailable = $qsVersionInfo->versionStatus == 'deprecated';
		if ( array_key_exists( 'UpdatePostponed', $qsConfig ) ) {
			$updateAvailable = $updateAvailable && $qsVersionInfo->actualVersion != $qsConfig['UpdatePostponed'];
		}
		return $updateAvailable;
	}

	/**
	 * @return bool Erfolg
	 */
	public function sendReport() {
		$config = $this->getConfiguration();
		$report = $this->report();
		$headers = 	'Content-Type: text/plain; charset=UTF-8'."\r\n";

		return mail( $config['Quickstart']['ReportRecipient'], 'FaxAPI QuickStart Testbericht', $report, $headers );
	}

	public function report() {
		$definitions = $this->getDefinitions();
		$config = &$this->getConfiguration();
		$report =	'QuickStart Version: v'.$config['Quickstart']['Version']."\n\n";

		if ( $this->isConfigured() ) {
			$report.=	"=== Definitionen ===\n";
			foreach ( $definitions as $key => $value ) {
				$report .= $key . ': ' . $value . "\n";
			}
		}
		else {
			$report.= 'Konfiguration: N/A'."\n";
		}
		$diagnostics = $this->getDiagnostics();
		$report .= $diagnostics->report();

		return $report;
	}
	
	public function &getDefinitions() {
		return $this->definitions;
	}
	
	public function isConfigured() {
		$definitions = &$this->getDefinitions();
		return array_key_exists( 'FAX_AUTHKEY', $definitions ) && strlen( $definitions['FAX_AUTHKEY'] ) > 0;
	}

	/**
	 * @return the $diagnostics
	 */
	public function getDiagnostics() {
		return $this->diagnostics;
	}
	
	public function getConfigurationFile() {
		return $this->configurationFile;
	}
	
	public function readConfiguration() {
		$this->configuration = self::readConfigurationFile( $this->getConfigurationFile() );
	}
	
	public function writeConfiguration() {
		self::writeConfigurationFile( $this->getConfigurationFile(), $this->getConfiguration() );
	}
	
	public static function &readConfigurationFile( $file ) {
		$configuration = parse_ini_file( $file, true );
		return $configuration;
	}
	
	public static function writeConfigurationFile( $file, &$configuration, $comment = null ) {
        $handle = fopen( $file, 'w' );
        if ( ! is_null( $comment ) ) {
        	fwrite( $handle, $comment );
        }
		foreach ( $configuration as $section => $elem ) { 
		    fwrite( $handle, "[".$section."]\n" );
            foreach ( $elem as $key => $value ) { 
                if( is_array( $value ) ) { 
                    for( $i=0; $i<count($value); $i++ ) { 
					    fwrite($handle, $key.'[' .$i . '] = "' .$value[$i] . "\"\n" );
                    } 
                } 
                else if( is_null( $value ) ) {
                	fwrite( $handle, $key." = \n" );	
                } 
                else {
                	fwrite( $handle, $key . ' = "' . $value . "\"\n" ); 
            	}                	
            } 
            fwrite( $handle, "\n" );
        } 
	}
	
	public function readDefinitions() {
		$config = &$this->getConfiguration();
		$definitionsFile = $config['ServiceExamples']['Definitions'];
		$this->definitions = self::readDefinitionsFile( $definitionsFile );
	}

	public static function &readDefinitionsFile( $definitionsFile ) {
		$definitions = array();
		if ( file_exists( $definitionsFile ) ) { 
			$content = file_get_contents( $definitionsFile );

			preg_match_all( self::PATTERN_DEFINE, $content, $matches );
			foreach( $matches[1] as $index => $key ) {
				$definitions[$key] = $matches[2][$index];
			}
		}
		return $definitions;
	}
	
	/**
	 * Enter description here ...
	 * @param string $authkey
	 * @param string $endpointId
	 */
	public function updateDefinitions( $authkey, $endpointId ) {
		$config = &$this->getConfiguration();
		$definitions = &$this->getDefinitions();
		$diagnostics = $this->getDiagnostics();
		$endpointGroup = $diagnostics->getEndpointGroup( $endpointId );
		
		$definitions['INCLUDE_PATH'] = realpath( $config['Quickstart']['IncludePath'] ) . DIRECTORY_SEPARATOR;
		$definitions['FAX_AUTHKEY']  = $authkey;
		$definitions['FAX_JOB_WSDL'] =  $endpointGroup->getEndpoint( 'Job' )->getServiceDescriptor();
		$definitions['FAX_SENDFAX_WSDL'] = $endpointGroup->getEndpoint( 'SendFax' )->getServiceDescriptor();
	}

	/**
	 * @param string 		$authKey
	 * @param string|null	$ignoreVersion
	 * @return bool
	 */
	public function writeDefinitions() {
		$config = &$this->getConfiguration();
		$comment = sprintf( "// Generiert von Fax-API QuickStart v%s am %s Uhr \n", $config['Version']['Present'], date('d.m.Y \u\m H:i') ) . "\n";
		$comment .= "// Änderungen an dieser Datei werden von QuickStart überschrieben.\n\n";
		self::writeDefinitionsFile( $this->getDefinitions(), $config['ServiceExamples']['Definitions'], $comment );
	}

	public static function writeDefinitionsFile( &$definitions, $definitionsFile, $comment = null ) {
		$handle = fopen( $definitionsFile, 'w');
		if ( ! $handle ) {
			throw new ErrorException( sprintf( 'Failed to write definitions [%s] unexpectedly.', $definitionsFile ) );
		}
		fwrite( $handle, "<?php\n" );
		if ( ! is_null( $comment ) ) {
			fwrite( $handle, $comment );
		}
		foreach( $definitions as $key => $value ) {
			$key = is_string( $key ) ? $key : $key->__toString();
			$value = is_string( $value ) ? $value : $value->__toString();
			fwrite( $handle, "define( '". addslashes( $key ) . "', '" . addslashes( $value ) . "' );\n" );
		}
		fclose( $handle );
	}

}